void radio_mute(int fd)
{   struct video_audio vid_aud;
    if (ioctl(fd, VIDIOCGAUDIO, &vid_aud)) perror("VIDIOCGAUDIO");
    vid_aud.flags |= VIDEO_AUDIO_MUTE;
    if (ioctl(fd, VIDIOCSAUDIO, &vid_aud)) perror("VIDIOCSAUDIO");
}

void radio_unmute(int fd)
{   struct video_audio vid_aud;
    if (ioctl(fd, VIDIOCGAUDIO, &vid_aud)) perror("VIDIOCGAUDIO");
    if (vid_aud.volume == 0) vid_aud.volume = 65535;
    vid_aud.flags &= ~VIDEO_AUDIO_MUTE;
    if (ioctl(fd, VIDIOCSAUDIO, &vid_aud)) perror("VIDIOCSAUDIO");
}

static int get_freq_fact(int fd)
{   struct video_tuner tuner;
    tuner.tuner = 0;
    if (ioctl (fd, VIDIOCGTUNER, &tuner) < 0) return 16;
    if ((tuner.flags & VIDEO_TUNER_LOW) == 0) return 16;
    return 16000;
}

int radio_setfreq(int fd, float freq)
{   int ifreq = (freq+1.0/32)*get_freq_fact(fd);
    return ioctl(fd, VIDIOCSFREQ, &ifreq);
}

void init_radio (void)
{   if (-1 == (fd = open(RADIO_DEVICE, O_RDONLY)))
    { perror("can't open " RADIO_DEVICE);
      exit(1);
    }
}

void scroll_tune (GtkAdjustment *adj)
{	radio_unmute(fd);
	radio_setfreq (fd, adj->value);
        sprintf(freq_string, "%3.2f", adj->value);
	freq_c = adj->value;
        gtk_label_set(GTK_LABEL (frequency), freq_string);
}

void scroll_volume (GtkAdjustment *adj)
{	mixer = open(MIXER_DEVICE, O_RDWR);
	volume = adj->value/-1;
	volume |= volume << 8;
	ioctl(mixer, MIXER_WRITE(SOUND_MIXER_VOLUME), &volume);
}

void set_volume(vol)
{	mixer = open(MIXER_DEVICE, O_RDWR);
	vol |= vol << 8;
	ioctl(mixer, MIXER_WRITE(SOUND_MIXER_VOLUME), &vol);
}

void get_volume (void)
{	mixer = open(MIXER_DEVICE, O_RDWR);
	ioctl(mixer, MIXER_READ(SOUND_MIXER_VOLUME), &volume);
	volume = ((volume & 0x7f) + ((volume >> 8) & 0x7f))/2;
	volume = volume/-1;
        gtk_adjustment_set_value(GTK_ADJUSTMENT (adj_volume), volume);
}

void status_toggle(GtkWidget *widget, gpointer data)
{   if (GTK_TOGGLE_BUTTON (mute_toggle)->active)
         set_volume(0);
    else
        set_volume(volume); 
}
					    
void tune_down (void)
{ 	freq_c -= STEPS;
	//tasten_freq[taste_aktuell] = freq_c;
	if (freq_c < MIN_FREQ) 
		freq_c = MAX_FREQ;
	radio_unmute(fd);
	radio_setfreq(fd, freq_c);
	sprintf(freq_string, "%3.2f", freq_c);
        gtk_label_set(GTK_LABEL (frequency), freq_string);
        gtk_adjustment_set_value(GTK_ADJUSTMENT (adj_tune), freq_c);
}

void tune_up (void)
{	freq_c += STEPS;
	//tasten_freq[taste_aktuell] = freq_c;    
	if (freq_c > MAX_FREQ) 
	 	freq_c = MIN_FREQ; 
	radio_unmute(fd); 
	radio_setfreq(fd, freq_c);
	sprintf(freq_string, "%3.2f", freq_c);
	gtk_label_set(GTK_LABEL (frequency), freq_string);
	gtk_adjustment_set_value(GTK_ADJUSTMENT (adj_tune), freq_c);
}

void quit_mute (void)
{ 	radio_mute(fd);
	close(fd);
	gtk_main_quit();
}

void write_to_rc (void)
{       int i;
        sprintf(path_rc, "%.199s/.rico_RADIOrc", getenv("HOME"));
 	config_file = fopen(path_rc, "w+");
	fprintf (config_file, "# This is the config-file for rico_RADIO\n");
	fprintf (config_file, "[saved frequencies]\n");
        for (i=1;i<=6;	fprintf (config_file, "%d = %3.2f\n", i,tasten_freq[i]),i++);
	fclose (config_file);
        config_file = fopen(path_rc, "r");
}

void taste (int i)
{	sprintf(freq_string, "%3.2f", tasten_freq[i]);
        gtk_label_set (GTK_LABEL (frequency), freq_string);
        gtk_adjustment_set_value(GTK_ADJUSTMENT (adj_tune), tasten_freq[i]);
        radio_setfreq(fd, tasten_freq[i]);
	freq_c = tasten_freq[i]; taste_aktuell = i;
}


void lade_freq(void)
{       int n;
        sprintf(path_rc, "%.199s/.rico_RADIOrc", getenv("HOME"));
        if (NULL == (config_file = fopen(path_rc, "r"))) write_to_rc();

        do
	{	fgets(path_rc,80,config_file);
		if (2 == sscanf(path_rc,"%d = %f",&n,&freq) && ((n > 0) & (n < 7))) tasten_freq[n] =  freq;
    	}
	while (!feof(config_file));
	fclose(config_file);
        write_to_rc();   
        taste(1);       
}

void stat_1 (void) { taste(1); }
void stat_2 (void) { taste(2); }
void stat_3 (void) { taste(3); }
void stat_4 (void) { taste(4); }
void stat_5 (void) { taste(5); }
void stat_6 (void) { taste(6); }
